/* 
   Smoke test for map

   Author: Bo Simonsen, December 2008
*/

#include "stl_map.h++"
#include "avl_tree.h++"
#include "node_iterator.h++"
#include "assert.h++"
#include <cassert>
#include <iostream>

template <typename K, typename V>
class functor {
  public:
    K const& operator()(V const& v) {
      return v.first;
    }
};

#define _avl_tree_map(K, V) cphstl::avl_tree<K,\
std::pair<const K, V>,\
functor<K, std::pair<const K, V> >,\
std::less<K>,\
std::allocator<std::pair<const K, V> >,\
cphstl::avl_tree_node< std::pair<const K, V> >,\
false>

#define _map(K, V, R, N) cphstl::map<K,\
V,\
std::less<K>,\
std::allocator<std::pair<const K, V> >,\
R,\
N>

#define _node_iterator(R) cphstl::node_iterator<R::node_type, false>,\
cphstl::node_iterator<R::node_type, true>


int main() {
  #ifndef STD
  typedef _map(int, char, _avl_tree_map(int, char), _node_iterator(_avl_tree_map(int, char))) R;
  #else
  typedef cphstl::map<int, char> R;
  #endif
  R m;

  assert(m.empty());

  m.insert(std::pair<int, char>(1, 'a'));
  m.insert(std::pair<int, char>(2, 'b'));

  assert(m.size() == 2);
  assert((*m.begin()).second == 'a');
  assert((*(--m.end())).first == 2);

  m[4] = 'd';

  assert(m.size() == 3);
  assert((*m.lower_bound(3)).first == 4);
  assert((*m.upper_bound(2)).first == 4);

  m.erase(1);
  assert(m.size() == 2);
  assert((*m.begin()).second == 'b');

  R m2(m);

  assert(!m2.empty());
  assert(m == m2);
  /* Additional check */
  assert(std::equal(m2.rbegin(), m2.rend(), m.rbegin()));

  assert(m2[2] == 'b');
  assert((*m2.lower_bound(1)).first == 2);

  std::cout << "0" << std::endl;

  assert(m2[1] == char());
  assert((*m2.begin()).first == 1);
  assert((*(--m2.rend())).first == 1);

  assert(m2.size() == 3);

  for(R::iterator it = m2.begin(); it != m2.end(); ++it) {
    std::cout << "Item: " << (*it).first << std::endl;
  }
  for(R::reverse_iterator it = m2.rbegin(); it != m2.rend(); ++it) {
    std::cout << "Item: " << (*it).first << std::endl;
  }

  std::cout << "1" << std::endl;

  m2.erase(m2.begin());
  assert(m2.size() == 2);

  m.erase(m.begin());
  assert(m.size() == 1);
  assert((*m.begin()).first == 4);

  for(R::iterator it = m2.begin(); it != m2.end(); ++it) {
    std::cout << "Item: " << (*it).first << std::endl;
  }
  for(R::reverse_iterator it = m2.rbegin(); it != m2.rend(); ++it) {
    std::cout << "Item: " << (*it).first << std::endl;
  }

  std::cout << "2" << std::endl;

  m2[5] = 'q';
  m2[9] = 'w';

  assert(m2.size() == 4);
  assert(*m2.rbegin() == *(--m2.end()));
  std::cout << (*m2.begin()).first << std::endl;
  std::cout << (*(++m2.begin())).first << std::endl;
  std::cout << (*(--(++m2.begin()))).first << std::endl;
  assert(--(++m2.begin()) != m2.end());
  assert(--(++m2.begin()) == m2.begin());
  std::cout << (*(--m2.rend())).first << std::endl;
  assert(*(--m2.rend()) == *m2.begin());
}
