/*
  This regression test for <set> was taken from the book
  [P.J. Plauger, Alexander A. Stepanov, Meng Lee, and David R.
  Musser, The C++ Standard Template Library, Prentice Hall PTR
  (2001)].
*/

#ifdef STD 
#include <set> 
#define KERNEL , std::set<char, Mypred, Myal> 
#else
#define KERNEL 
#endif

#include <memory>
#include <cassert>
#include <iostream> 
#include <functional> 
#include <utility>
#include "stl_set.h++"

template <typename T>
inline void ignore_unused_variable_warning(const T&) {
}

void test_set() {
  typedef std::allocator<char> Myal;
  typedef std::less<char> Mypred; 
  typedef cphstl::set<char, Mypred, Myal KERNEL > Mycont; 
  char ch, carr[] = "abc", carr2[] = "def"; 
  
  Mycont::key_type* p_key = (char*) 0; 
  ignore_unused_variable_warning(p_key);
  Mycont::key_compare* p_kcomp = (Mypred*) 0; 
  ignore_unused_variable_warning(p_kcomp);
  Mycont::value_type* p_val = (char*) 0;
  ignore_unused_variable_warning(p_val); 
  Mycont::value_compare* p_vcomp = (Mypred*) 0;
  ignore_unused_variable_warning(p_vcomp); 
  Mycont::allocator_type* p_alloc = (Myal*) 0;
  ignore_unused_variable_warning(p_alloc); 
  Mycont::pointer p_ptr = (char*) 0; 
  ignore_unused_variable_warning(p_ptr);
  Mycont::const_pointer p_cptr = (const char*) 0;
  ignore_unused_variable_warning(p_cptr); 
  Mycont::reference p_ref = ch; 
  ignore_unused_variable_warning(p_ref);
  Mycont::const_reference p_cref = (const char&)ch; 
  ignore_unused_variable_warning(p_cref);
  Mycont::size_type* p_size = (size_t*) 0;
  ignore_unused_variable_warning(p_size); 
  Mycont::difference_type* p_diff = (ptrdiff_t*) 0; 
  ignore_unused_variable_warning(p_diff);

  Mycont v0; 
  Myal al = v0.get_allocator(); 
  Mypred pred; 
  Mycont v0a(pred), v0b(pred, al); 
  assert(v0.empty() && v0.size() == 0); 
  assert(v0a.size() == 0 && v0a.get_allocator() == al); 
  assert(v0b.size() == 0 && v0b.get_allocator() == al); 
  Mycont v1(carr, carr + 3); 
  assert(v1.size() == 3 && *v1.begin() == 'a'); 
  Mycont v2(carr, carr + 3, pred); 
  assert(v2.size() == 3 && *v2.begin() == 'a'); 
  Mycont v3(carr, carr + 3, pred, al); 
  assert(v3.size() == 3 && *v3.begin() == 'a'); 
  const Mycont v4(carr, carr + 3); 
  v0 = v4; 
  assert(v0.size() == 3 && *v0.begin() == 'a'); 
  
  Mycont::iterator p_it(v1.begin()); 
  Mycont::const_iterator p_cit(v4.begin()); 
  Mycont::reverse_iterator p_rit(v1.rbegin()); 
  Mycont::const_reverse_iterator p_crit(v4.rbegin()); 
  assert(*p_it == 'a' && *--(p_it = v1.end()) == 'c'); 
  assert(*p_cit == 'a' && *--(p_cit = v4.end()) == 'c'); 
  assert(*p_rit == 'c' && *--(p_rit = v1.rend()) == 'a'); 
  assert(*p_crit == 'c' && *--(p_crit = v4.rend()) == 'a'); 
  v0.clear(); // Differs from multiset 
  assert(v0.size() == 0);
  std::pair<Mycont::iterator, bool> pib = v0.insert('d'); 
  assert(*pib.first == 'd' && pib.second); 
  
  assert(*--v0.end() == 'd'); 
  pib = v0.insert('d'); 
  assert(*pib.first == 'd' && !pib.second); 
  assert(*v0.insert(v0.begin() , 'e') == 'e'); 
  v0.insert(carr, carr + 3); 
  assert(v0.size() == 5 && *v0.begin() == 'a'); 
  v0.insert(carr2, carr2 + 3); 
  assert(v0.size() == 6 && *--v0.end() == 'f'); 
  v0.erase(v0.begin()); // Standard ambiguous: return value of erase?
  assert(v0.size() == 5);  // *v0.erase(v0.begin()) == 'b' 
  v0.erase(v0.begin(), ++v0.begin());
  assert(v0.size() == 4); 
  assert(v0.erase('x') == 0 && v0.erase('e') == 1); 

  v0.clear(); 
  assert(v0.empty()); 
  v0.swap(v1); 
  assert(!v0.empty() && v1.empty()); 
  swap(v0, v1); 
  assert(v0.empty() && !v1.empty()); 
  assert(v1 == v1 && v0 < v1); 
  assert(v0 != v1 && v1 > v0); 
  assert(v0 <= v1 && v1 >= v0); 
  
  assert(v0.key_comp()('a', 'c') && !v0.key_comp()('a', 'a')); 
  assert(v0.value_comp()('a', 'c') && !v0.value_comp()('a', 'a')); 
  assert(*v4.find('b') == 'b'); 
  assert(v4.count('x') == 0 && v4.count('b') == 1); 
  assert(*v4.lower_bound('a') == 'a'); 
  assert(*v4.upper_bound('a') == 'b'); 
  std::pair<Mycont::const_iterator, Mycont::const_iterator> pcc = v4.equal_range('a'); 
  assert(*pcc.first == 'a' && *pcc.second == 'b'); 
} 

int main() {
  test_set();
  return (0); 
}
