/*
  A smoke test for cphstl::stack

  Authors: Jyrki Katajainen and Johannes Serup, March 2008
*/

#include <cassert>
#include "stack.h++"

template <typename V>
void test_stack() {
  typedef cphstl::stack<V> stack;
  typedef typename stack::iterator iterator;
  typedef typename stack::const_iterator const_iterator;
  typedef typename stack::const_reverse_iterator const_reverse_iterator;

  stack s1;
  stack s2;
  assert(s1.size() == 0);
  assert(s2.empty());
  s1.push(1);
  assert(s1.size() == 1);
  s2.push(1);

  // Test '=='
  assert(s1 == s2);

  // Test copy constructor
  stack s3(s2);
  assert(s2 == s3);

  // Test '='
  s1 = s2;
  assert(s1 == s2);
  assert(s2 == s3);

  // Test '<'
  s2.push(3);	
  assert(s1 < s2);
	
  // Test '>'
  assert(s2 > s1);

  // Test clear
  s1.clear();
  assert(s1.empty());

  // Test top
  for (V i = 1; i <= 10; ++i) {
    s1.push(i);
  }	  
  V v1 = s1.top();
  assert(v1 == 10);
  s1.top() = 11;
  v1 = s1.top();
  assert(v1 == 11);
  V const v2 = s2.top();
  s1.top() = 10;
  assert(v2 == 3);
  assert(*s1.begin() == s1.top());
  assert(*s2.begin() == s2.top());
  assert(*s3.begin() == s3.top());

  // Test begin and end
  iterator it1 = s1.begin();
  iterator it2 = --s1.end();
  assert(*it1 == 10);
  assert(*it2 == 1);

  // Test const_iterator
  V j = 10;
  for (const_iterator t = s1.begin(); t != s1.end(); ++t) {
    assert(*t == j);
    --j;
  }

  // Test const_reverse_iterator
  V i = 1;
  for (const_reverse_iterator t = s1.rbegin(); t != s1.rend(); ++t) {
    assert(*t == i);
    ++i;
  }

  // Test max_size
  assert(s1.size() < s1.max_size());

  // Test const_iterator
  const_iterator it = s1.begin();
  assert(*it == 10);
  // *it = 1; compilation should fail!

  // Test pop
  s1.pop();
  assert(s1.top() == 9);

  // Test swap
  swap(s1, s2);
  assert(s1.size() == 2);
  assert(s2.size() == 9);
}

int main(int, char**) {
  test_stack<int>();
  test_stack<char>();
  return 0;
}
