/*
  The dynamic_assert macro is taken quite directly from the book
  [Steve Maguire, Writing solid code, Microsoft Press (1993)]. The
  macro is a statement that cannot be used in expression context.

  The static_assert macro is taken from the book [Andrei Alexandrescu,
  Modern C++ design: Generic programming and design patterns applied,
  Addison-Wesley (2001)].

  Author: Jyrki Katajainen, December 2001, April 2008
*/

#ifndef __CPHSTL_ASSERT__
#define __CPHSTL_ASSERT__

#include <cstdio> // defines std::fflush, std::fprintf
#include <cstdlib> // defines std::abort

namespace cphstl {

#ifdef NDEBUG
#define dynamic_assert(condition)
#else

  void __assert(char const* message, char const* strFile, unsigned int uLine) {
    std::fflush(NULL);
    std::fprintf(stderr, "\nAssertion `%s´ failed: %s, line %u\n",
		 message, strFile, uLine);
    std::fflush(stderr);
    std::abort();
  } 

#define dynamic_assert(condition) \
    if (condition) { \
    } \
    else \
      cphstl::__assert(#condition, __FILE__, __LINE__)
#endif

  template<bool>
  class compile_time_checker {
  public:
    compile_time_checker(...) {
    }
  };

  template<>
  class compile_time_checker<false> {
  };

#define static_assert(condition, message) { \
    class ERROR_##message { \
    }; \
    typedef cphstl::compile_time_checker<(condition)> type; \
    type temp = type(ERROR_##message()); \
    (void) sizeof(temp); \
  }

}

#endif
