/*
  Implementation of cphstl::entry_iterator

  Author: Jyrki Katajainen, Bo Simonsen, April 2008

  General design idea is proposed by J. Katajainen and B. Simonsen
*/

#include <cassert> // defines assert macro
#include <iostream>

namespace cphstl {

  // default constructor

  template <typename R, typename E, bool is_const>
  entry_iterator<R, E, is_const>::entry_iterator() 
    : position(node_pointer()) 
  {
  }
  
  // copy constructor
  
  template <typename R, typename E, bool is_const>
  entry_iterator<R, E, is_const>::entry_iterator(entry_iterator<R, E, false> const& a) 
    : position(a.position) {
  }
   template <typename R, typename E, bool is_const>
  entry_iterator<R, E, is_const>::entry_iterator(entry_iterator<R, E, true> const& a) 
    : position(a.position) {
  }
  
  // assignment

  template <typename R, typename E, bool is_const>
  entry_iterator<R, E, is_const>&
  entry_iterator<R, E, is_const>::operator=(entry_iterator<R, E, is_const> const& a) {
    (*this).position = a.position;
    return *this;
  }
  
  // destructor
  
  template <typename R, typename E, bool is_const>
  entry_iterator<R, E, is_const>::~entry_iterator() {
  }

  // operator*  

  template <typename R, typename E, bool is_const>
  typename entry_iterator<R, E, is_const>::reference
  entry_iterator<R, E, is_const>::operator*() const {
    return reference((*(*this).position).content());
  }
  
  // operator->  

  template <typename R, typename E, bool is_const>
  typename entry_iterator<R, E, is_const>::pointer
  entry_iterator<R, E, is_const>::operator->() const {
    return pointer(&(*(*this).position).content());
  }
  
  // operator++; pre-increment
  
  template <typename R, typename E, bool is_const>
  entry_iterator<R, E, is_const>&
  entry_iterator<R, E, is_const>::operator++() {
    (*this).position = (*(*this).position).advance(1);
    return *this;
  }
  
  // operator++; post-increment

  template <typename R, typename E, bool is_const>
  entry_iterator<R, E, is_const>
  entry_iterator<R, E, is_const>::operator++(int) {
    entry_iterator<R, E, is_const> temporary = *this;
    (*this).position = (*(*this).position).advance(1);
    return temporary;
  }
  
  // operator--; pre-decrement
  
  template <typename R, typename E, bool is_const>
  entry_iterator<R, E, is_const>&
  entry_iterator<R, E, is_const>::operator--() {
    (*this).position = (*(*this).position).advance(-1);
    return *this;
  }
  
  // operator--; post-decrement
  
  template <typename R, typename E, bool is_const>
  entry_iterator<R, E, is_const>
  entry_iterator<R, E, is_const>::operator--(int) {
    entry_iterator<R, E, is_const> temporary = *this;
    (*this).position = (*(*this).position).advance(-1);
    return temporary;
  }
  
  // operator+=
  
  template <typename R, typename E, bool is_const>
  entry_iterator<R, E, is_const>&
  entry_iterator<R, E, is_const>::operator+=(difference_type n) {
    (*this).position = (*(*this).position).advance(n);
    return *this;
  }
  
  // operator-=
  
  template <typename R, typename E, bool is_const>
  entry_iterator<R, E, is_const>&
  entry_iterator<R, E, is_const>::operator-=(difference_type n) {
    (*this).position = (*(*this).position).advance(-n);
    return *this;
  }
  
  // operator+

  template <typename R, typename E, bool is_const>
  entry_iterator<R, E, is_const>
  entry_iterator<R, E, is_const>::operator+(difference_type n) const {
    entry_iterator<R, E, is_const> temporary = *this;
    temporary.position = (*temporary.position).advance(n);
    return temporary;
  }
  
  // operator-
  
  template <typename R, typename E, bool is_const>
  entry_iterator<R, E, is_const>
  entry_iterator<R, E, is_const>::operator-(difference_type n) const {
    entry_iterator<R, E, is_const> temporary = *this;
    temporary.position = (*temporary.position).advance(-n);
    return temporary;
  }
  
  // iterator distance
  
  template <typename R, typename E, bool is_const>
  typename entry_iterator<R, E, is_const>::difference_type
  entry_iterator<R, E, is_const>::operator-(entry_iterator<R, E, is_const> const& a) const {
    return (*(*this).position).distance(a.position);
  }
  
  // operator==

  template <typename R, typename E, bool is_const>
  template <bool both>
  bool
  entry_iterator<R, E, is_const>::operator==(entry_iterator<R, E, both> const& a) const {
    return (*this).position == a.position;
  }
  
  // operator!=

  template <typename R, typename E, bool is_const>
  template <bool both>
  bool
  entry_iterator<R, E, is_const>::operator!=(entry_iterator<R, E, both> const& a) const {
    return !(*this == a);
  }
  
  // operator<

  template <typename R, typename E, bool is_const>
  template <bool both>
  bool
  entry_iterator<R, E, is_const>::operator<(entry_iterator<R, E, both> const& a) const {
    return (*(*this).position).distance(a.position) < 0;
  }
  
  // operator>

  template <typename R, typename E, bool is_const>
  template <bool both>
  bool
  entry_iterator<R, E, is_const>::operator>(entry_iterator<R, E, both> const& a) const {
    return a < *this;
  }
  
  // operator<=

  template <typename R, typename E, bool is_const>
  template <bool both>
  bool
  entry_iterator<R, E, is_const>::operator<=(entry_iterator<R, E, both> const& a) const {
    return !(a < *this);
  }
  
  // operator>=

  template <typename R, typename E, bool is_const>
  template <bool both>
  bool
  entry_iterator<R, E, is_const>::operator>=(entry_iterator<R, E, both> const& a) const {
    return !(*this < a);
  }
  
  // operator+(int, iterator)

  template <typename R, typename E, bool is_const>
  entry_iterator<R, E, is_const> operator+(typename R::difference_type n, entry_iterator<R, E, is_const> const& a) {
    return a + n;
  }

  // parametrized constructor
  
  template <typename R, typename E, bool is_const>
  entry_iterator<R, E, is_const>::entry_iterator(node_pointer const& _position) 
    : position(_position) {
  }
  
  // convertion operator

  template <typename R, typename E, bool is_const>
  entry_iterator<R, E, is_const>::operator node_pointer() const {
    return (*this).position;
  }
  
}
