namespace cphstl {

  template <typename V>
  class list_node {
  public:

    // types

    typedef V value_type;
    typedef list_node node_type;
    typedef list_node* link_type;

    // structors

    list_node();
    list_node(V const&, list_node* = 0, list_node* = 0);
    ~list_node();

    // accessors

    list_node* predecessor() const;
    list_node* successor() const;
    V const& content() const;

    // modifiers

    list_node*& predecessor();
    list_node*& successor();
    V& content();

  private:

    list_node* previous;
    list_node* next;
    V value;

    // unsupported operations

    list_node(list_node const&);
    list_node& operator=(list_node const&);
    
  };

}

#include "list_node.c++" // implements cphstl::list_node

#if defined(UNITTEST_LIST_NODE)

#include <cassert> // defines assert

template <typename T>
void test_list_node() {

  cphstl::list_node<T>* p = new cphstl::list_node<T>(T(1));
  cphstl::list_node<T>* q = new cphstl::list_node<T>(T(2), p);
  cphstl::list_node<T>* r = new cphstl::list_node<T>(T(3), q);
  cphstl::list_node<T>* past_the_end = new cphstl::list_node<T>();

  (*p).predecessor() = p;
  (*p).successor() = q;
  (*q).successor() = r;
  (*r).successor() = past_the_end;
  (*past_the_end).predecessor() = r;
  (*past_the_end).successor() = past_the_end;

  assert((*p).content() == T(1));
  assert((*q).content() == T(2));
  assert((*r).content() == T(3));
  assert((*p).predecessor() == p);
  assert((*p).successor() == q);
  assert((*q).predecessor() == p);
  assert((*q).successor() == r);
  assert((*r).predecessor() == q);
  assert((*r).successor() == past_the_end);
  assert((*past_the_end).predecessor() == r);
  assert((*past_the_end).successor() == past_the_end);

  (*r).content() = T(4);
  assert((*r).content() == T(4));
}

int main(int, char**) {
  test_list_node<int>();
  test_list_node<char>();
  return 0;
}

#endif
