/*
  Implementation of cphstl::node_iterator

  Author: Jyrki Katajainen, Bo Simonsen, April 2008

  General design idea is proposed by J. Katajainen and B. Simonsen
*/

#include <sstream> // defines std::stringstream

namespace cphstl {

  // default constructor

  template <typename N, bool is_const>
  node_iterator<N, is_const>::node_iterator() 
    : link(0) {
  }
  
  // copy constructor
  
  template <typename N, bool is_const>
  node_iterator<N, is_const>::node_iterator(node_iterator<N, false> const& a) 
    : link(a.link) {
  }

  // assignment

  template <typename N, bool is_const>
  node_iterator<N, is_const>&
  node_iterator<N, is_const>::operator=(node_iterator<N, is_const> const& a) {
    link = a.link;
    return *this;
  }
  
  // destructor
  
  template <typename N, bool is_const>
  node_iterator<N, is_const>::~node_iterator() {
  }

  // operator*  

  template <typename N, bool is_const>
  typename node_iterator<N, is_const>::reference
  node_iterator<N, is_const>::operator*() const {
    return reference((*link).content());
  }
  
  // operator->  

  template <typename N, bool is_const>
  typename node_iterator<N, is_const>::pointer
  node_iterator<N, is_const>::operator->() const {
    return pointer(&(*link).content());
  }
  
  // operator++; pre-increment
  
  template <typename N, bool is_const>
  node_iterator<N, is_const>&
  node_iterator<N, is_const>::operator++() {
    link = (*link).successor();
    return *this;
  }
  
   // operator++; post-increment

  template <typename N, bool is_const>
  node_iterator<N, is_const>
  node_iterator<N, is_const>::operator++(int) {
    node_iterator<N, is_const> temporary(*this);
    ++(*this);
    return temporary;
  }
 
  // operator--; pre-increment
  
  template <typename N, bool is_const>
  node_iterator<N, is_const>&
  node_iterator<N, is_const>::operator--() {
    link = (*link).predecessor();
    return *this;
  }
  
  // operator--; post-increment

  template <typename N, bool is_const>
  node_iterator<N, is_const>
  node_iterator<N, is_const>::operator--(int) {
    node_iterator<N, is_const> temporary(*this);
    --(*this);
    return temporary;
  }
  
  // operator==

  template <typename N, bool is_const>
  template <bool both>
  bool
  node_iterator<N, is_const>::operator==(node_iterator<N, both> const& a) const {
    return link == a.link;
  }
  
  // operator!=

  template <typename N, bool is_const>
  template <bool both>
  bool
  node_iterator<N, is_const>::operator!=(node_iterator<N, both> const& a) const {
    return link != a.link;
  }
  
  // parametrized constructor (node -> iterator)
  
  template <typename N, bool both>
  node_iterator<N, both>::node_iterator(node_pointer p) 
    : link(p) {
  }
  
  // conversion operators (iterator -> node)

  template <typename N, bool is_const>
  node_iterator<N, is_const>::operator node_pointer() const {
    return link;
  }

  // conversion operator (makes it possible to print out an iterator)

  template <typename N, bool is_const>
  node_iterator<N, is_const>::operator std::string() const {
    std::stringstream ss;
    std::string address;
    ss << (int)(char*)((*this).link);
    ss >> address;

    if (is_const == false) {
      return std::string("iterator: node at ") + address;
    }
    else {
      return std::string("const_iterator: node at ") + address;
    }
  }

  // representation

  template <typename N, bool both>
  std::ostream& 
  operator<<(std::ostream& s, node_iterator<N, both> const& i) {
    s  << std::string(i);
    return s;
  }

}
