/*
  An implementation of cphstl::stack

  Authors: Jyrki Katajainen and Johannes Serup, March 2008
*/

#include <algorithm> // defines std::equal and std::lexicographical_compare

namespace cphstl {

  // explicit constructor

  template <typename V, typename A, typename R>
  stack<V, A, R>::stack(A const& a) : kernel(a) {
  }

  // copy constructor

  template <typename V, typename A, typename R>
  stack<V, A, R>::stack(stack<V, A, R> const& s) : kernel(s.kernel) {
  }

  // destructor

  template <typename V, typename A, typename R>
  stack<V, A, R>::~stack() {
  }

  // operator=

  template <typename V, typename A, typename R>
  stack<V, A, R>&
  stack<V, A, R>::operator=(stack<V, A, R> const& s) {
    (*this).kernel = s.kernel;
    return *this;
  }

  // begin

  template <typename V, typename A, typename R>
  typename stack<V, A, R>::iterator
  stack<V, A, R>::begin() {
    return (*this).kernel.begin();
  }
	
  template <typename V, typename A, typename R>
  typename stack<V, A, R>::const_iterator
  stack<V, A, R>::begin() const {
    return const_iterator((*this).kernel.begin());
  }
	
  // end

  template <typename V, typename A, typename R>
  typename stack<V, A, R>::iterator
  stack<V, A, R>::end() {
    return (*this).kernel.end();
  }	
	
  template <typename V, typename A, typename R>
  typename stack<V, A, R>::const_iterator
  stack<V, A, R>::end() const {
    return const_iterator((*this).kernel.end());
  }
	
  // rbegin

  template <typename V, typename A, typename R>
  typename stack<V, A, R>::reverse_iterator
  stack<V, A, R>::rbegin() {
    return reverse_iterator((*this).kernel.end());
  }
  
  template <typename V, typename A, typename R>
  typename stack<V, A, R>::const_reverse_iterator
  stack<V, A, R>::rbegin() const {		
    return const_reverse_iterator((*this).kernel.end());
  }
  
  // rend

  template <typename V, typename A, typename R>
  typename stack<V, A, R>::reverse_iterator
  stack<V, A, R>::rend() {
    return reverse_iterator((*this).kernel.begin());
  }
	
  template <typename V, typename A, typename R>
  typename stack<V, A, R>::const_reverse_iterator
  stack<V, A, R>::rend() const {
    return const_reverse_iterator((*this).kernel.begin());
  }
	
  // size

  template <typename V, typename A, typename R>
  typename stack<V, A, R>::size_type
  stack<V, A, R>::size() const {
    return (*this).kernel.size();
  }
	
  // max_size

  template <typename V, typename A, typename R>
  typename stack<V, A, R>::size_type
  stack<V, A, R>::max_size() const {
    return (*this).kernel.max_size();
  }
  
  // empty

  template <typename V, typename A, typename R>
  bool
  stack<V, A, R>::empty() const {
    return (*this).size() == 0;
  }
	
  // top

  template <typename V, typename A, typename R>
  V&
  stack<V, A, R>::top() {
    return (*this).kernel.top();
  }
	
  template <typename V, typename A, typename R>
  V const&
  stack<V, A, R>::top() const {
    return (*this).kernel.top();    	
  }
	
  // push

  template <typename V, typename A, typename R>
  void
  stack<V, A, R>::push(V const& v) {
    (*this).kernel.push(v);
  }
	
  // pop

  template <typename V, typename A, typename R>
  void
  stack<V, A, R>::pop() {
    (*this).kernel.pop();
  }

  // clear

  template <typename V, typename A, typename R>
  void
  stack<V, A, R>::clear() {
    (*this).kernel.clear();
  }
  
  // swap

  template <typename V, typename A, typename R>
  void
  stack<V, A, R>::swap(stack<V, A, R>& s) {
    (*this).kernel.swap(s.kernel);
  }
	
  template <typename V, typename A, typename R>
  void
  swap(stack<V, A, R>& x, stack<V, A, R>& y) {
    x.swap(y);
  }

  // operator==

  template <typename V, typename A, typename R>
  bool
  operator==(stack<V, A, R> const& x, stack<V, A, R> const& y) {
    return (x.size() == y.size() && std::equal(x.begin(), x.end(), y.begin()));
  }
  
  // operator<

  template <typename V, typename A, typename R>
  bool
  operator<(stack<V, A, R> const& x, stack<V, A, R> const& y) {
    return std::lexicographical_compare(x.begin(), x.end(), y.begin(), y.end());
  }
  
  // operator!=

  template <typename V, typename A, typename R>
  bool
  operator!=(stack<V, A, R> const& x, stack<V, A, R> const& y) {
    return !(x == y);
  }

  // operator>
  
  template <typename V, typename A, typename R>
  bool
  operator>(stack<V, A, R> const& x, stack<V, A, R> const& y) {
    return (y < x);
  }

  // operator>=

  template <typename V, typename A, typename R>
  bool
  operator>=(stack<V, A, R> const& x, stack<V, A, R> const& y) {
    return !(x < y);
  }

  // operator<=

  template <typename V, typename A, typename R>
  bool
  operator<=(stack<V, A, R> const& x, stack<V, A, R> const& y) {
    return !(y < x);
  }
  
}
