/* 
   Implementation of the cphstl::map bridge class,
   most origins from the cphstl::set implementation
   
   Author: Bo Simonsen, December 2008
*/

namespace cphstl {
  // explicit constructor

  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  map<K, V, C, A, R, I, J>::map(C const& comparator, A const& allocator) 
    : map_base<K, V, C, A, R, I, J>(comparator, allocator) {
  }

  // parametrized constructor

  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  template <typename X>
  map<K, V, C, A, R, I, J>::map(X p, X q, C const& comparator, A const& allocator)
    : map_base<K, V, C, A, R, I, J>(p, q, comparator, allocator) {
  }

  // copy constructor
  
  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  map<K, V, C, A, R, I, J>::map(map const& other) : map_base<K, V, C, A, R, I, J>(other) {
  }

  // destructor
  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  map<K, V, C, A, R, I, J>::~map() {
  }

  // operator=
  
  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  map<K, V, C, A, R, I, J>& 
  map<K, V, C, A, R, I, J>::operator=(map const& other) {
    (*this).kernel = other.kernel;
    return *this;
  }

  // operator[]
  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  typename map<K, V, C, A, R, I, J>::mapped_type&
  map<K, V, C, A, R, I, J>::operator[](key_type const& k) {
    iterator i = (*this).lower_bound(k);

    if(i == (*this).end() || (*this).key_comp()(k, (*i).first)) {
      i = (*this).insert(i, value_type(k, mapped_type()));
    }

    return (*i).second;
  }


  // count

  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  typename map<K, V, C, A, R, I, J>::size_type
  map<K, V, C, A, R, I, J>::count(K const& k) const {
    if ((*this).find(k) == (*this).end()) {
      return size_type(0); 
    }
    return size_type(1);
  }
  // single-element insert

  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  std::pair<typename map<K, V, C, A, R, I, J>::iterator, bool>
  map<K, V, C, A, R, I, J>::insert(value_type const& e) {
    return (*this).kernel.insert(e);
  }

  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  typename map<K, V, C, A, R, I, J>::iterator
  map<K, V, C, A, R, I, J>::insert(iterator p, value_type const& e) {
    return (*this).kernel.insert(p, e); // _unique
  }

  // multiple-element insert

  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  template <typename X>
  void
  map<K, V, C, A, R, I, J>::insert(X p, X q) {
    (*this).kernel.insert(p, q); // _unique
  }


  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  void 
  swap(map<K, V, C, A, R, I, J>& r, map<K, V, C, A, R, I, J>& s) {
    r.swap(s);
  }

  /* Casting operator for printing out the realisation data structure

  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  map<K, V, C, A, R, I, J>::operator std::string() const {
    return std::string((*this).kernel);
  } */
 
  // operator==

  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  bool
  operator==(map<K, V, C, A, R, I, J> const& r, map<K, V, C, A, R, I, J> const& s) {
    return (r.size() == s.size() && std::equal(r.begin(), r.end(), s.begin()));
  }

  // operator!=

  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  bool
  operator!=(map<K, V, C, A, R, I, J> const& r, map<K, V, C, A, R, I, J> const& s) {
    return !(r == s);
  }

  // operator<

  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  bool
  operator<(map<K, V, C, A, R, I, J> const& r, map<K, V, C, A, R, I, J> const& s) {
    return std::lexicographical_compare(r.begin(), r.end(), s.begin(), s.end(),
					r.key_comp());
  }

  // operator>

  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  bool
  operator>(map<K, V, C, A, R, I, J> const& r, map<K, V, C, A, R, I, J> const& s) {
    return (s < r);
  }

  // operator<=

  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  bool
  operator<=(map<K, V, C, A, R, I, J> const& r, map<K, V, C, A, R, I, J> const& s) {
    return !(s < r);
  }

  // operator>=

  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  bool
  operator>=(map<K, V, C, A, R, I, J> const& r, map<K, V, C, A, R, I, J> const& s) {
    return !(r < s);
  }


}
