/* 
   Bridge class for cphstl::map, almost directly taken from the C++ standard.

   Author: Bo Simonsen, December 2008
*/

#include <map>
#include "stl_set_base.h++"

#ifndef CPHSTL_MAP_BASE_H
#define CPHSTL_MAP_BASE_H

namespace cphstl {
  template <typename K, 
            typename V, 
            typename C = std::less<K>,
            typename A = std::allocator<std::pair<const K, V> >,
            typename R = std::map<K, V, C, A>,
            typename I = typename R::iterator,
            typename J = typename R::const_iterator
           >

  class map_base : public set_base<K, V, C, A, R, I, J> {
  public:
    typedef I iterator;
    typedef J const_iterator;    
    typedef K key_type;        
 
    // structors
  
    explicit map_base(C const& = C(), A const& = A());
  
    template <typename X>
    map_base(X, X, C const& = C(), A const& = A());
  
    map_base(map_base const&);
    ~map_base();
   
    iterator lower_bound(const  key_type& x);
    const_iterator lower_bound(const  key_type& x) const;
    iterator upper_bound(const  key_type& x);
    const_iterator upper_bound(const  key_type& x) const;

    std::pair<iterator,iterator> equal_range(const key_type& x);
    std::pair<const_iterator,const_iterator> equal_range(const key_type& x) const;
  };
}

#include "stl_map_base.c++"
#endif
