/* 
   Implementation of the cphstl::multimap bridge class,
   most origins from the cphstl::set implementation
   
   Author: Bo Simonsen, December 2008
*/

namespace cphstl {
  // explicit constructor

  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  multimap<K, V, C, A, R, I, J>::multimap(C const& comparator, A const& allocator) 
    : map_base<K, V, C, A, R, I, J>(comparator, allocator) {
  }

  // parametrized constructor

  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  template <typename X>
  multimap<K, V, C, A, R, I, J>::multimap(X p, X q, C const& comparator, A const& allocator)
    : map_base<K, V, C, A, R, I, J>(p, q, comparator, allocator) {
  }

  // copy constructor
  
  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  multimap<K, V, C, A, R, I, J>::multimap(multimap const& other) : map_base<K, V, C, A, R, I, J>(other) {
  }

  // destructor
  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  multimap<K, V, C, A, R, I, J>::~multimap() {
  }

  // operator=
  
  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  multimap<K, V, C, A, R, I, J>& 
  multimap<K, V, C, A, R, I, J>::operator=(multimap const& other) {
    (*this).kernel = other.kernel;
    return *this;
  }

  // count

  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  typename multimap<K, V, C, A, R, I, J>::size_type
  multimap<K, V, C, A, R, I, J>::count(K const& k) const {
    if ((*this).find(k) == (*this).end()) {
      return size_type(0); 
    }
    return size_type(std::distance((*this).lower_bound(k), 
                                   (*this).upper_bound(k)));

  }
  // single-element insert

  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  typename multimap<K, V, C, A, R, I, J>::iterator
  multimap<K, V, C, A, R, I, J>::insert(value_type const& e) {
    return (*this).kernel.insert(e);
  }

  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  typename multimap<K, V, C, A, R, I, J>::iterator
  multimap<K, V, C, A, R, I, J>::insert(iterator p, value_type const& e) {
    return (*this).kernel.insert(p, e); // _unique
  }

  // multiple-element insert

  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  template <typename X>
  void
  multimap<K, V, C, A, R, I, J>::insert(X p, X q) {
    (*this).kernel.insert(p, q); // _unique
  }


  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  void 
  swap(multimap<K, V, C, A, R, I, J>& r, multimap<K, V, C, A, R, I, J>& s) {
    r.swap(s);
  }

  /* Casting operator for printing out the realisation data structure

  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  multimap<K, V, C, A, R, I, J>::operator std::string() const {
    return std::string((*this).kernel);
  } */
 
  // operator==

  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  bool
  operator==(multimap<K, V, C, A, R, I, J> const& r, multimap<K, V, C, A, R, I, J> const& s) {
    return (r.size() == s.size() && std::equal(r.begin(), r.end(), s.begin()));
  }

  // operator!=

  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  bool
  operator!=(multimap<K, V, C, A, R, I, J> const& r, multimap<K, V, C, A, R, I, J> const& s) {
    return !(r == s);
  }

  // operator<

  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  bool
  operator<(multimap<K, V, C, A, R, I, J> const& r, multimap<K, V, C, A, R, I, J> const& s) {
    return std::lexicographical_compare(r.begin(), r.end(), s.begin(), s.end(),
					r.key_comp());
  }

  // operator>

  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  bool
  operator>(multimap<K, V, C, A, R, I, J> const& r, multimap<K, V, C, A, R, I, J> const& s) {
    return (s < r);
  }

  // operator<=

  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  bool
  operator<=(multimap<K, V, C, A, R, I, J> const& r, multimap<K, V, C, A, R, I, J> const& s) {
    return !(s < r);
  }

  // operator>=

  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  bool
  operator>=(multimap<K, V, C, A, R, I, J> const& r, multimap<K, V, C, A, R, I, J> const& s) {
    return !(r < s);
  }


}
