/* 
   Bridge class for cphstl::map, almost directly taken from the C++ standard.

   Author: Bo Simonsen, December 2008
*/

#include <map>
#include "stl_map_base.h++"

#ifndef CPHSTL_MAP_H
#define CPHSTL_MAP_H

namespace cphstl {
  template <typename K, 
            typename V, 
            typename C = std::less<K>,
            typename A = std::allocator<std::pair<const K, V> >,
            typename R = std::map<K, V, C, A>,
            typename I = typename R::iterator,
            typename J = typename R::const_iterator
           >

  class map : public map_base<K, V, C, A, R, I, J> {
  public:
    // types:
    typedef K key_type;
    typedef V mapped_type;
    typedef std::pair<const K, V> value_type;
    typedef C key_compare;
    typedef A allocator_type;
    typedef I iterator;       
    typedef J const_iterator; 

    typedef typename R::reference reference;
    typedef typename R::const_reference const_reference;
    typedef typename R::pointer pointer;
    typedef typename R::const_pointer const_pointer;

    typedef typename R::value_compare value_compare;

    typedef std::size_t size_type;
    typedef std::ptrdiff_t difference_type;
    typedef std::reverse_iterator<iterator> reverse_iterator;
    typedef std::reverse_iterator<const_iterator> const_reverse_iterator;

    explicit map(C const& = C(), A const& = A());

    template <typename X>
    map(X p, X q, C const& c = C(), A const& a = A());
    map(const map<K, V, C, A, R, I, J>& x);

    ~map();

    map<K, V, C, A, R, I, J>& operator=(const map<K, V, C, A, R, I, J>& x);

    iterator insert(const value_type& x);
    iterator insert(iterator p, const value_type& x);

    template <typename X>
    void insert(X p, X q);

    size_type count(const key_type& x) const;
  };

  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
     bool operator==(const map<K,V,C,A,R,I,J>& x,
                            const map<K,V,C,A,R,I,J>& y);
  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
    bool operator< (const map<K,V,C,A,R,I,J>& x,
                            const map<K,V,C,A,R,I,J>& y);
  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
    bool operator!=(const map<K,V,C,A,R,I,J>& x,
                            const map<K,V,C,A,R,I,J>& y);
  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
    bool operator> (const map<K,V,C,A,R,I,J>& x,
                            const map<K,V,C,A,R,I,J>& y);
  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
    bool operator>=(const map<K,V,C,A,R,I,J>& x,
                            const map<K,V,C,A,R,I,J>& y);
  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
    bool operator<=(const map<K,V,C,A,R,I,J>& x,
                            const map<K,V,C,A,R,I,J>& y);
  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
    void swap(map<K,V,C,A,R,I,J>& x,
                    map<K,V,C,A,R,I,J>& y);

} // cphstl namespace

#include "stl_map.c++"
#endif
