/*
  This header is taken quite directly from the C++ standard [2003].
  According to the terminology of the standard, a set satisfies all of
  the requirements of a container, of a reversible container, and of
  an associative container; and provides bidirectional iterators to
  the elements stored.

  Author: Jyrki Katajainen, May 2007, March 2008
*/

#ifndef __CPHSTL_MULTISET__
#define __CPHSTL_MULTISET__

#include <set>
#include "stl_set_base.h++"

namespace cphstl {

  template <
    typename V, 
    typename C = std::less<V>,
    typename A = std::allocator<V>,
    typename R = std::multiset<V, C, A>,
    typename I = typename R::iterator, 
    typename J = typename R::const_iterator
  >
  class multiset : public set_base<V, V, C, A, R, I, J> {

  public:
    // types
  
    typedef V key_type;
    typedef V value_type;
    typedef C key_compare;
    typedef C value_compare;
    typedef A allocator_type;
    typedef V& reference;
    typedef V const& const_reference;
    typedef I iterator;
    typedef J const_iterator;
    typedef std::size_t size_type;
    typedef std::ptrdiff_t difference_type;
    typedef V* pointer;
    typedef V const* const_pointer;
    typedef std::reverse_iterator<iterator> reverse_iterator;
    typedef std::reverse_iterator<const_iterator> const_reverse_iterator;

    // structors
  
    explicit multiset(C const& = C(), A const& = A());
  
    template <typename K>
    multiset(K, K, C const& = C(), A const& = A());
  
    multiset(multiset const&);
    ~multiset();
 
    multiset& operator=(multiset const&);

    size_type count(V const&) const;

    // modifiers

    iterator insert(V const&);
    iterator insert(iterator, V const&);
    template <typename K>
    void insert(K, K);

    operator std::string() const;
  };

  template <typename V, typename C, typename A, typename R, typename I, typename J>
  void swap(multiset<V, C, A, R, I, J>&, multiset<V, C, A, R, I, J>&);
  
  template <typename V, typename C, typename A, typename R, typename I, typename J>
  bool operator==(multiset<V, C, A, R, I, J> const&, multiset<V, C, A, R, I, J> const&);
  
  template <typename V, typename C, typename A, typename R, typename I, typename J>
  bool operator<(multiset<V, C, A, R, I, J> const&, multiset<V, C, A, R, I, J> const&);

  template <typename V, typename C, typename A, typename R, typename I, typename J>
  bool operator!=(multiset<V, C, A, R, I, J> const&, multiset<V, C, A, R, I, J> const&);
  
  template <typename V, typename C, typename A, typename R, typename I, typename J>
  bool operator>(multiset<V, C, A, R, I, J> const&, multiset<V, C, A, R, I, J> const&);

  template <typename V, typename C, typename A, typename R, typename I, typename J>
  bool operator>=(multiset<V, C, A, R, I, J> const&, multiset<V, C, A, R, I, J> const&);

  template <typename V, typename C, typename A, typename R, typename I, typename J>
  bool operator<=(multiset<V, C, A, R, I, J> const&, multiset<V, C, A, R, I, J> const&);


}

#include "stl_multiset.c++" // implements cphstl::multiset

#endif
