/*
  A set is a bridge class that calls the functions available in the
  given realization.

  Author: Jyrki Katajainen, May 2007, March 2008
*/

namespace cphstl {

  // explicit constructor

  template <typename V, typename C, typename A, typename R, typename I, typename J>
  set<V, C, A, R, I, J>::set(C const& comparator, A const& allocator) 
    : set_base<V, V, C, A, R, I, J>(comparator, allocator) {
  }

  // parametrized constructor

  template <typename V, typename C, typename A, typename R, typename I, typename J>
  template <typename K>
  set<V, C, A, R, I, J>::set(K p, K q, C const& comparator, A const& allocator)
    : set_base<V, V, C, A, R, I, J>(p, q, comparator, allocator) {
  }

  // copy constructor

  template <typename V, typename C, typename A, typename R, typename I, typename J>
  set<V, C, A, R, I, J>::set(set const& other) : set_base<V, V, C, A, R, I, J>(other) {
  }

  // destructor

  template <typename V, typename C, typename A, typename R, typename I, typename J>
  set<V, C, A, R, I, J>::~set() {
  }

  // operator=

  template <typename V, typename C, typename A, typename R, typename I, typename J>
  set<V, C, A, R, I, J>& 
  set<V, C, A, R, I, J>::operator=(set const& other) {
    (*this).kernel = other.kernel;
    return *this;
  }


  // count

  template <typename V, typename C, typename A, typename R, typename I, typename J>
  typename set<V, C, A, R, I, J>::size_type
  set<V, C, A, R, I, J>::count(V const& k) const {
    if ((*this).find(k) == (*this).end()) {
      return size_type(0); 
    }
    return size_type(1);
  }

  // single-element insert

  template <typename V, typename C, typename A, typename R, typename I, typename J>
  std::pair<typename set<V, C, A, R, I, J>::iterator, bool>
  set<V, C, A, R, I, J>::insert(V const& e) {
    return (*this).kernel.insert(e);
  }

  template <typename V, typename C, typename A, typename R, typename I, typename J>
  typename set<V, C, A, R, I, J>::iterator
  set<V, C, A, R, I, J>::insert(iterator p, V const& e) {
    return (*this).kernel.insert(p, e); // _unique
  }
  
  // multiple-element insert
  
  template <typename V, typename C, typename A, typename R, typename I, typename J>
  template <typename K>
  void
  set<V, C, A, R, I, J>::insert(K p, K q) {
    return (*this).kernel.insert(p, q);
  }

  template <typename V, typename C, typename A, typename R, typename I, typename J>
  void 
  swap(set<V, C, A, R, I, J>& r, set<V, C, A, R, I, J>& s) {
    r.swap(s);
  }


  // operator==

  template <typename V, typename C, typename A, typename R, typename I, typename J>
  bool
  operator==(set<V, C, A, R, I, J> const& r, set<V, C, A, R, I, J> const& s) {
    return (r.size() == s.size() && std::equal(r.begin(), r.end(), s.begin()));
  }

  // operator!=

  template <typename V, typename C, typename A, typename R, typename I, typename J>
  bool
  operator!=(set<V, C, A, R, I, J> const& r, set<V, C, A, R, I, J> const& s) {
    return !(r == s);
  }

  // operator<

  template <typename V, typename C, typename A, typename R, typename I, typename J>
  bool
  operator<(set<V, C, A, R, I, J> const& r, set<V, C, A, R, I, J> const& s) {
    return std::lexicographical_compare(r.begin(), r.end(), s.begin(), s.end(),
					r.key_comp());
  }

  // operator>

  template <typename V, typename C, typename A, typename R, typename I, typename J>
  bool
  operator>(set<V, C, A, R, I, J> const& r, set<V, C, A, R, I, J> const& s) {
    return (s < r);
  }

  // operator<=

  template <typename V, typename C, typename A, typename R, typename I, typename J>
  bool
  operator<=(set<V, C, A, R, I, J> const& r, set<V, C, A, R, I, J> const& s) {
    return !(s < r);
  }

  // operator>=

  template <typename V, typename C, typename A, typename R, typename I, typename J>
  bool
  operator>=(set<V, C, A, R, I, J> const& r, set<V, C, A, R, I, J> const& s) {
    return !(r < s);
  }

  template <typename V, typename C, typename A, typename R, typename I, typename J>
  set<V, C, A, R, I, J>::operator std::string() const {
    return std::string((*this).kernel);
  }
 
}
