/*
  A set_base is a bridge class that calls the functions available in the
  given realization.

  Author: Jyrki Katajainen, May 2007, March 2008
*/

namespace cphstl {

  // explicit constructor

  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  set_base<K, V, C, A, R, I, J>::set_base(C const& comparator, A const& allocator) 
    : kernel(comparator, allocator) {
  }

  // parametrized constructor

  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  template <typename X>
  set_base<K, V, C, A, R, I, J>::set_base(X p, X q, C const& comparator, A const& allocator)
    : kernel(comparator, allocator) {
    (*this).kernel.insert(p, q);
  }

  // copy constructor

  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  set_base<K, V, C, A, R, I, J>::set_base(set_base const& other) : kernel(other.kernel) {
  }

  // destructor

  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  set_base<K, V, C, A, R, I, J>::~set_base() {
  }

  // operator=

  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  set_base<K, V, C, A, R, I, J>& 
  set_base<K, V, C, A, R, I, J>::operator=(set_base const& other) {
    (*this).kernel = other.kernel;
    return *this;
  }

  // begin

  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  typename set_base<K, V, C, A, R, I, J>::iterator 
  set_base<K, V, C, A, R, I, J>::begin() {
    return iterator((*this).kernel.begin());
  }

  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  typename set_base<K, V, C, A, R, I, J>::const_iterator 
  set_base<K, V, C, A, R, I, J>::begin() const {
    return const_iterator((*this).kernel.begin());
  }

  // end

  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  typename set_base<K, V, C, A, R, I, J>::iterator
  set_base<K, V, C, A, R, I, J>::end() {
    return iterator((*this).kernel.end());
  }

  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  typename set_base<K, V, C, A, R, I, J>::const_iterator 
  set_base<K, V, C, A, R, I, J>::end() const {
    return const_iterator((*this).kernel.end());
  }

  // rbegin

  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  typename set_base<K, V, C, A, R, I, J>::reverse_iterator 
  set_base<K, V, C, A, R, I, J>::rbegin() {
    return reverse_iterator((*this).end());
  }

  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  typename set_base<K, V, C, A, R, I, J>::const_reverse_iterator 
  set_base<K, V, C, A, R, I, J>::rbegin() const {
    return const_reverse_iterator((*this).end());
  }

  // rend

  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  typename set_base<K, V, C, A, R, I, J>::reverse_iterator 
  set_base<K, V, C, A, R, I, J>::rend() {
    return reverse_iterator((*this).begin());
  }

  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  typename set_base<K, V, C, A, R, I, J>::const_reverse_iterator 
  set_base<K, V, C, A, R, I, J>::rend() const {
    return const_reverse_iterator((*this).begin());
  }

  // get allocator

  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  A
  set_base<K, V, C, A, R, I, J>::get_allocator() const {
    return (*this).kernel.get_allocator();
  }

  // get comparator

  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  typename set_base<K, V, C, A, R, I, J>::key_compare 
  set_base<K, V, C, A, R, I, J>::key_comp() const {
    return (*this).kernel.key_comp();
  }

  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  typename set_base<K, V, C, A, R, I, J>::value_compare 
  set_base<K, V, C, A, R, I, J>::value_comp() const {
    return (*this).kernel.value_comp();
  }

  // empty

  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  bool 
  set_base<K, V, C, A, R, I, J>::empty() const {
    return (*this).size() == size_type(0);
  }

  // size

  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  typename set_base<K, V, C, A, R, I, J>::size_type 
  set_base<K, V, C, A, R, I, J>::size() const {
    return (*this).kernel.size();
  }

  // max_size

  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  typename set_base<K, V, C, A, R, I, J>::size_type 
  set_base<K, V, C, A, R, I, J>::max_size() const {
    return (*this).kernel.max_size();
  }

  // find

  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  typename set_base<K, V, C, A, R, I, J>::iterator
  set_base<K, V, C, A, R, I, J>::find(K const& k) {
    iterator p = (*this).lower_bound(k);
    cphstl::unnamed::key_map<K, V> key;
    key_compare comp = (*this).key_comp();
    
    if(p == (*this).end() || comp(k, key[*p])) {
      return (*this).end();
    }
      
    return p; 
  }

  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  typename set_base<K, V, C, A, R, I, J>::const_iterator
  set_base<K, V, C, A, R, I, J>::find(K const& k) const {
    const_iterator p = (*this).lower_bound(k);
    cphstl::unnamed::key_map<K, V> key;
    key_compare comp = (*this).key_comp();
    
    if(p == (*this).end() || comp(k, key[*p])) {
      return (*this).end();
    }
      
    return p; 
  }

  // lower_bound

  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  typename set_base<K, V, C, A, R, I, J>::iterator
  set_base<K, V, C, A, R, I, J>::lower_bound(K const& k) const {
    return (*this).kernel.lower_bound(k);
  }

  // upper_bound

  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  typename set_base<K, V, C, A, R, I, J>::iterator
  set_base<K, V, C, A, R, I, J>::upper_bound(K const& k) const {
    return (*this).kernel.upper_bound(k);
  }

  // equal_range

  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  std::pair<typename set_base<K, V, C, A, R, I, J>::iterator, 
	    typename set_base<K, V, C, A, R, I, J>::iterator>
  set_base<K, V, C, A, R, I, J>::equal_range(K const& k) const {
    return std::make_pair((*this).lower_bound(k), (*this).upper_bound(k));
  }

  // single-element erase

  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  typename set_base<K, V, C, A, R, I, J>::size_type
  set_base<K, V, C, A, R, I, J>::erase(K const& e) {
    return (*this).kernel.erase(e);
  }

  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  void
  set_base<K, V, C, A, R, I, J>::erase(iterator p) {
    (*this).kernel.erase(p);
  }

  // multiple-element erase

  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  void
  set_base<K, V, C, A, R, I, J>::erase(iterator p, iterator s) {
    for (iterator r = p; r != s;) {
      iterator q = r;
      ++r;
      (*this).kernel.erase(q);
    }
  }

  // clear

  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  void 
  set_base<K, V, C, A, R, I, J>::clear() {
    (*this).erase((*this).begin(), (*this).end());
  }

  // swap

  template <typename K, typename V, typename C, typename A, typename R, typename I, typename J>
  void 
  set_base<K, V, C, A, R, I, J>::swap(set_base<K, V, C, A, R, I, J>& r) {
    (*this).kernel.swap(r.kernel);
  }


}
