#ifndef _CPHSTL_STL_BASE_H
#define _CPHSTL_STL_BASE_H

#include <functional> // defines std::less
#include <iterator> // defines std::reverse_iterator
#include <memory> // defines std::allocator
#include <utility> // defines std::pair
#include "set_helper.c++" // defines cphstl::unnamed::{identity and key_map}

namespace cphstl {

  template <
    typename K,
    typename V, 
    typename C,
    typename A,
    typename R,
    typename I, 
    typename J
  >
  class set_base {

  public:
    // types
  
    typedef K key_type;
    typedef V value_type;
    typedef C key_compare;
    typedef C value_compare;
    typedef A allocator_type;
    typedef V& reference;
    typedef V const& const_reference;
    typedef I iterator;
    typedef J const_iterator;
    typedef std::size_t size_type;
    typedef std::ptrdiff_t difference_type;
    typedef V* pointer;
    typedef V const* const_pointer;
    typedef std::reverse_iterator<iterator> reverse_iterator;
    typedef std::reverse_iterator<const_iterator> const_reverse_iterator;

    // structors
  
    explicit set_base(C const& = C(), A const& = A());
  
    template <typename X>
    set_base(X, X, C const& = C(), A const& = A());
  
    set_base(set_base const&);
    ~set_base();
    set_base& operator=(set_base const&);
  
    // iterators

    iterator begin();
    const_iterator begin() const;
    iterator end();
    const_iterator end() const;
    reverse_iterator rbegin();
    const_reverse_iterator rbegin() const;
    reverse_iterator rend();
    const_reverse_iterator rend() const;

    // accessors

    A get_allocator() const;
    C key_comp() const;
    C value_comp() const;
    bool empty() const;
    size_type size() const;
    size_type max_size() const;
    iterator find(K const&);
    const_iterator find(K const&) const;    
    iterator lower_bound(K const&) const;
    iterator upper_bound(K const&) const;
    std::pair<iterator, iterator> equal_range(K const&) const;

    size_type erase(K const&); 
    void erase(iterator);
    void erase(iterator, iterator);
    void clear();
    void swap(set_base&);

  protected:

    typedef R realization_type;
    realization_type kernel;

  };

}

#include "stl_set_base.c++"
#endif
