/*
  Original author: Jyrki Katajainen, December 2001. Revised May 2006.
*/

#include <cstddef> // defines std::size_t

namespace cphstl {

  template <int v>
  class int2type {
  public:
    enum { value = v };
  };

  template <bool v>
  class bool2type {
  public:
    enum { value = v };
  };


  template <typename T, typename U>
  class if_then_else<true, T, U> {
  public:
    typedef T type;
  };

  template <typename T, typename U>
  class if_then_else<false, T, U> {
  public:
    typedef U type;
  };

  template <typename X, typename Y>
  class same_traits {
  public:
    enum { positive = 0 };
    enum { negative = 1 };
  };

  template <typename X>
  class same_traits <X, X> {
  public:
    enum { positive = 1 };
    enum { negative = 0 };
  };

  template <typename U>
  class non_const {
  public:
    typedef U type;
  };

  template <typename U>
  class non_const <U const> {
  public:
    typedef U type;
  };

  template <typename U>
  class non_volatile {
  public:
    typedef U type;
  };

  template <typename U>
  class non_volatile <volatile U> {
  public:
    typedef U type;
  };

  template <typename U>
  class reference_traits { 
  public:
    typedef U& type;
    enum { positive = 0};
  };

  template <typename U>
  class reference_traits <U&> {
  public: 
    typedef U& type;
    enum { positive = 1};
  };

  template <>
  class reference_traits <void> {
  public: 
    typedef void type;
    enum {positive = 0};
  };

  template <typename U>
  class const_reference_traits { 
  public:
    typedef U const& type;
  };

  template <typename U>
  class const_reference_traits <U const> {
  public: 
    typedef U const& type;
  };

  template <typename U>
  class const_reference_traits <U&> {
  public: 
    typedef U const& type;
  };

  template <>
  class const_reference_traits <void> {
  public: 
    typedef void type;
  };

  template <typename U>
  class pointer_traits {
  public:
    enum { positive = 0 };
  };

  template <typename U>
  class pointer_traits <U*> {
  public:
    enum { positive = 1 };
  };

  template <typename T>
  class lookup_traits {
  public:
    enum { 
      unsigned_integral = 0,
      signed_integral = 0,
      other_integral = 0,
      floating_point = 0
    };
  };

  template <>
  class lookup_traits <bool> {
  public:
    enum {
      unsigned_integral = 0,
      signed_integral = 0,
      other_integral = 1,
      floating_point = 0
    };
  };

  template <>
  class lookup_traits <char> {
  public:
    enum {
      unsigned_integral = 0,
      signed_integral = 0,
      other_integral = 1,
      floating_point = 0
    };
  };

  template <>
  class lookup_traits <signed char> {
  public:
    enum {
      unsigned_integral = 0,
      signed_integral = 1,
      other_integral = 0,
      floating_point = 0
    };
  };

  template <>
  class lookup_traits <unsigned char> {
  public:
    enum {
      unsigned_integral = 1,
      signed_integral = 0,
      other_integral = 0,
      floating_point = 0
    };
  };

  template <>
  class lookup_traits <wchar_t> {
  public:
    enum {
      unsigned_integral = 0,
      signed_integral = 0,
      other_integral = 1,
      floating_point = 0
    };
  };

  template <>
  class lookup_traits <signed short> {
  public:
    enum {
      unsigned_integral = 0,
      signed_integral = 1,
      other_integral = 0,
      floating_point = 0
    };
  };

  template <>
  class lookup_traits <unsigned short> {
  public:
    enum {
      unsigned_integral = 1,
      signed_integral = 0,
      other_integral = 0,
      floating_point = 0
    };
  };

  template <>
  class lookup_traits <signed int> {
  public:
    enum {
      unsigned_integral = 0,
      signed_integral = 1,
      other_integral = 0,
      floating_point = 0
    };
  };

  template <>
  class lookup_traits <unsigned int> {
  public:
    enum {
      unsigned_integral = 1,
      signed_integral = 0,
      other_integral = 0,
      floating_point = 0
    };
  };

  template <>
  class lookup_traits <signed long> {
  public:
    enum {
      unsigned_integral = 0,
      signed_integral = 1,
      other_integral = 0,
      floating_point = 0
    };
  };

  template <>
  class lookup_traits <unsigned long> {
  public:
    enum {
      unsigned_integral = 1,
      signed_integral = 0,
      other_integral = 0,
      floating_point = 0
    };
  };

#if LONGLONG_EXISTS
  template <>
  class lookup_traits <signed long long> {
  public:
    enum {
      unsigned_integral = 0,
      signed_integral = 1,
      other_integral = 0,
      floating_point = 0
    };
  };

  template <>
  class lookup_traits <unsigned long long> {
  public:
    enum {
      unsigned_integral = 1,
      signed_integral = 0,
      other_integral = 0,
      floating_point = 0
    };
  };
#endif

  template <>
  class lookup_traits <float> {
  public:
    enum {
      unsigned_integral = 0,
      signed_integral = 0,
      other_integral = 0,
      floating_point = 1
    };
  };

  template <>
  class lookup_traits <double> {
  public:
    enum {
      unsigned_integral = 0,
      signed_integral = 0,
      other_integral = 0,
      floating_point = 1
    };
  };

  template <>
  class lookup_traits <long double> {
  public:
    enum {
      unsigned_integral = 0,
      signed_integral = 0,
      other_integral = 0,
      floating_point = 1
    };
  };

  template <typename T>
  class array_traits {
  public:
    enum { positive = 0};
  };

  template <typename T, std::size_t N>
  class array_traits <T[N]> {
  public:
    enum { positive = 1};
  };

  template <typename T>
  class array_traits <T[]> {
  public:
    enum { positive = 1};
  };

  template <typename T>
  class function_traits {
  private:
    typedef char one;
    typedef struct {char a[2]; } two;
    template <typename U> 
    static one test(...);
    template <typename U> 
    static two test(U (*)[1]);
  public:
    enum { positive = sizeof(function_traits<T>::template test<T>(0)) == 1 };
  };

  template <typename T>
  class function_traits <T&> {
  public:
    enum { positive = 0 };
  };

  template <>
  class function_traits <void> {
  public:
    enum { positive = 0 };
  };

  template <>
  class function_traits <void const> {
  public:
    enum { positive = 0 };
  };

  template <>
  class function_traits <void volatile> {
  public:
    enum { positive = 0 };
  };

  template <>
  class function_traits <void const volatile> {
  public:
    enum { positive = 0 };
  };

  template <typename T>
  class pointer_to_member_traits {
  public:
    enum { positive = 0 };
  };

  template <typename T, typename C>
  class pointer_to_member_traits <T C::*> {
  public:
    enum { positive = 1 };
  };

  template <typename T>
  class class_traits {
  private:
    typedef char one;
    typedef struct {char a[2]; } two;
    template <typename U> 
    static one test(int U::*);
    template <typename U> 
    static two test(...);
  public:
    enum { positive = sizeof(class_traits<T>::template test<T>(0)) == 1 };
  };

  template <typename T>
  class member_const_iterator {
  private:
    typedef char one;
    typedef struct {char a[2]; } two;
    template <typename U> 
    static one test(typename U::const_iterator const*);
    template <typename U> 
    static two test(...);
  public:
    enum { positive = sizeof(test<T>(0)) == 1 };
  };
}

