/* -*- C++ -*-

Author: Jyrki Katajainen, December 2001. Revised May 2006.


This module is a CPH STL extension which provides a collection of
tools for performing type mappings. The purpose of this module is to
make the library self-contained, but these tools can be useful in
other contexts as well.

The main sources when writing this have been
 
 Andrei Alexandrescu. Modern C++ Design: Generic Programming and
 Design Patterns Applied, Addison-Wesley (2001), see Chapters 2 and 3.

 Jaakko Jrvi, Tuple types and multiple return values, C/C++ Users
 Journal 19,8 (2001), 24-35.

 David Vandevoorde and Nicolai M. Josuttis. C++ Templates: The
 Complete Guide. Addison-Wesley (2003), see Sections 13.10 and 15.2
 and Chapter 19.

Status: No unittesting done.
*/

#ifndef __CPHSTL_TYPE__
#define __CPHSTL_TYPE__

namespace cphstl {

  template <int v>
  class int2type;

  template <bool V>
  class bool2type;

  template <bool, typename T, typename U>
  class if_then_else;

}

#include "type.c++" // Implements the type functions

namespace cphstl {

  template <typename T, typename U>
  class types {
  public:
    enum {
      are_same = cphstl::same_traits<T, U>::positive,
      are_not_same = cphstl::same_traits<T, U>::negative
    };
  };

  template <typename T>
  class type {
  public:
    typedef T original_type;
    typedef typename non_volatile<typename non_const<T>::type>::type unqualified;
    typedef const typename cphstl::non_const<T>::type const_qualified;
    typedef volatile typename cphstl::non_volatile<T>::type volatile_qualified;
    typedef const volatile typename cphstl::type<T>::unqualified const_volatile_qualified;
    typedef typename cphstl::non_const<T>::type non_const;
    typedef typename non_volatile<T>::type non_volatile;
    typedef typename cphstl::reference_traits<T>::type reference;
    typedef typename cphstl::const_reference_traits<T>::type const_reference;
    typedef T* pointer;
    typedef T* const const_pointer;

    enum {
      is_unsigned_integral = cphstl::lookup_traits<T>::unsigned_integral,
      is_signed_integral = cphstl::lookup_traits<T>::signed_integral,
      is_other_integral = cphstl::lookup_traits<T>::other_integral,
      is_integral = is_unsigned_integral || is_signed_integral || 
                    is_other_integral,
      is_floating_point = cphstl::lookup_traits<T>::floating_point,
      is_arithmetic = is_integral || is_floating_point,
      is_fundamental = is_arithmetic || is_floating_point || 
                       cphstl::types<T, void>::are_same,
      is_pointer = cphstl::pointer_traits<T>::positive,
      is_reference = cphstl::reference_traits<T>::positive,
      is_array = cphstl::array_traits<T>::positive,
      is_function = cphstl::function_traits<T>::positive,
      is_pointer_to_member = cphstl::pointer_to_member_traits<T>::positive,
      //      is_enumeration = ?,
      //      is_union = ?,
      //      is_bit_copiable = ?,
      is_class = cphstl::class_traits<T>::positive,
	  has_member_const_iterator = cphstl::member_const_iterator<T>::positive
    };
  };	

  /* To be included:

  Author: efim.slobodov233432@arcor.de, April 6th, 2005
  Source: http://www.codecomments.com/archive324-2005-4-448963.html

typedef char Small;
typedef struct {char unused[2];} Big;

template<int> 
class Selector;

template <typename T> 
struct Has_member_dual {
private:
  template <typename U>
  static ::Small test(::Selector<sizeof(&U::dual)>*);

  template <typename U>
  static ::Big test(...);

public:
  enum{value=sizeof(test<T>(0))==sizeof(::Small)};
};

// test

class my1 {
};

class my2 {
public:
  void dual();
};

#include <iostream>

int main(int, char **) {
  std::cout << "my1=" << Has_member_dual<my1>::value << "== 0?" 
            << std::endl;
  std::cout << "my2=" << Has_member_dual<my2>::value << "== 1?" 
            << std::endl;
  std::cout << "int=" << Has_member_dual<int>::value << "== 0?" 
            << std::endl;
  return 0;
} 
*/
}

#endif // __CPHSTL_TYPE__



