/*
  The assert macro is taken quite directly from the book [Steve
  Maguire, Writing solid code, Microsoft Press (1993)]. This macro is
  a statement that cannot be used in expression context.

  The static assert is now provided by the compiler.

  Author: Jyrki Katajainen © 2001, 2008
*/

#ifndef __CPHSTL_ASSERT__
#define __CPHSTL_ASSERT__

#include <cstdio> // std::fflush, std::fprintf
#include <cstdlib> // std::abort

namespace cphstl {

#ifdef NDEBUG
#define assert(condition)
#else

  void __assert(char const* message, char const* strFile, unsigned int uLine) {
    std::fflush(NULL);
    std::fprintf(stderr, "\nAssertion '%s' failed: %s, line %u\n",
		 message, strFile, uLine);
    std::fflush(stderr);
    std::abort();
  } 

#define assert(condition) \
  if (condition) { \
  } \
  else \
    cphstl::__assert(#condition, __FILE__, __LINE__)
#endif

}

#endif
