#ifndef __CPHSTL_ENCAPSULATOR_FACTORY__
#define __CPHSTL_ENCAPSULATOR_FACTORY__

/*
  The factory class templates.  We provide two versions of this class,
  a general version which is used to create indirect/doubly-indirect
  encapsulators and a specialized version for the direct encapsulator.
  These classes ensure that the creation of objects is done in a
  generic way with respect to the framework class.

  Authors: Jyrki Katajainen, Bo Simonsen © 2008
*/

#include "allocator-proxy.h++"

namespace cphstl {

  template <typename V, typename A>
  class direct_encapsulator; 

  template <typename V, typename A, typename E>
  class encapsulator_factory {
  public:

    typedef E encapsulator_type;
    typedef typename encapsulator_type::value_type value_type;
    typedef typename A::template rebind<encapsulator_type>::other encapsulator_allocator_type;
    typedef std::size_t size_type;

    encapsulator_factory(A const& a = A()) : encapsulator_allocator(a) {
    }
    
    encapsulator_factory(encapsulator_factory const& f) : encapsulator_allocator(f.encapsulator_allocator) {
    }

    void create(value_type const& v, encapsulator_type*& e, size_type const& index) {
      e = encapsulator_allocator.allocate(1);
      new(e) encapsulator_type(v, encapsulator_allocator.subject(), index);
    }

    void destroy(encapsulator_type* entry) {
      (*entry).~encapsulator_type();
      encapsulator_allocator.deallocate(entry, 1);
    }

  private:

    allocator_proxy<encapsulator_allocator_type> encapsulator_allocator;
  };

  template <typename V, typename A>
  class encapsulator_factory <V, A, direct_encapsulator<V, A> > {
  public:

    typedef direct_encapsulator<V, A> encapsulator_type;
    typedef typename encapsulator_type::value_type value_type;
    typedef std::size_t size_type;

    encapsulator_factory(A const& a = A()) {
    }

    void create(value_type const& v, encapsulator_type& e, size_type const& index) {
      new (&e) encapsulator_type(v);
    }

    void destroy(encapsulator_type& e) {
      e.~encapsulator_type();
    }

  };
}
#endif
