#ifndef __CPHSTL_INDIRECT_ENCAPSULATOR__
#define __CPHSTL_INDIRECT_ENCAPSULATOR__

/*
  Defines an indirect encapsulator. 
  Segments store pointers to objects
  of the class indirect_encapsulator which store the value.

  Authors: Bo Simonsen, May 2008, September 2008
  Bue Krogh Vedel-Larsen and Mikkel Jønsson Thomsen, March 2009
*/

#include <cassert>
#include <memory.h>

#include "allocator-proxy.h++"

namespace cphstl {
  template <typename V, typename A>
  class indirect_encapsulator;

  template <typename V, typename A>
  void slot_swap(indirect_encapsulator<V, A>*&, indirect_encapsulator<V, A>*&);

  template <typename V, typename A>
  class indirect_encapsulator {
  private:
    typedef indirect_encapsulator<V, A> this_type;
    typedef typename A::template rebind<this_type>::other input_allocator_type;

  public:
    friend void slot_swap<>(indirect_encapsulator<V, A>*&, indirect_encapsulator<V, A>*&);

    typedef V value_type;
    typedef A allocator_type;
    typedef std::size_t size_type;
    typedef this_type** segment_type;
    typedef this_type* slot_type;

    enum {size = sizeof(this_type*) + 
                 sizeof(size_type) + 
                 sizeof(value_type)};
  public:

    ~indirect_encapsulator() {}
    indirect_encapsulator(value_type const& _v, input_allocator_type& _a, size_type const& _index) {
      (*this).v = _v;
      (*this).p = _index;
    }

    value_type & content() {
      return (*this).v;
    }
    value_type const& content() const {
      return (*this).v;
    }

    size_type position() const {
      return (*this).p;
    }

  private:
    value_type v;
    size_type p;


  };
}

#endif // __CPHSTL_SPDA_SP_DYNAMIC_ARRAY_ENTRY_H__
