#!/bin/python

# Preprocessor for named template arguments.
# The preprocessor should be invoked instead of g++.
# All arguments are given to g++.
# Made by Bo Simonsen <bo@geekworld.dk>, June 2009

import sys
import re
import os
import os.path
import time

include_dirs = []
included_files = []

p_include = re.compile('(#include (")([^ ]+)("))*')
p_namespace = re.compile(".*namespace(.*)$")
p_template = re.compile(".*template[ ]*<(.+)>[ ]*class ([^;:]+)")
p_parameter = re.compile('(typename|bool|int|char) ([^ ]+)[ ]*=?[ ]*(.*)')

gd = {} 

def parse_cpp_block(buf, l):
  def split_blocks(buf):
    blocks = []

    count = 0
    j = 0
    for i in xrange(0, len(buf)):
      if buf[i] == '<':
        count += 1
      if buf[i] == '>':
        count -= 1
      if buf[i] == ',' and count == 0:
        blocks.append(buf[j:i])
        j = i+1
      i += 1

    blocks.append(buf[j:])
  
    return blocks

  def transform_default_value((k,v)):
    i = v.find("<")
    j = v.rfind(">")

    if i == -1 and j == -1:
      return (k,(v, []))      

    args = v[i+1:j]
    new_v = [x.strip() for x in split_blocks(args)]

    return (k,(v[:i], new_v))

  d = {}

  i = -1
  j = -1

  while 1:
    old_j = j+1
  
    i = buf.find("{", i+1)
    if i == -1:
      break

    count = 1

    j = i
    while count > 0:
      if buf[j+1] == '{':
        count += 1 
      elif buf[j+1] == '}':
        count -= 1
      j+=1

    block = buf[i+1:j]
    newbuf = buf[old_j:i]

    if p_namespace.match(newbuf):
      tok = p_namespace.split(newbuf)[1].strip()
      l.append(tok)

    elif p_template.match(newbuf):
      arr = p_template.split(newbuf)
      class_name = arr[2].strip()
      blocks = [x.strip() for x in split_blocks(arr[1])]

      # We don't care about specialization!
      # i.e. class class_name<...>
      if class_name.find("<") == -1 and class_name.find(">") == -1:
        new_class_name = "::".join(l) + "::" + class_name
        new_blocks = [transform_default_value(tuple(p_parameter.split(x)[2:4])) for x in blocks]
        d[new_class_name] = new_blocks
    
    d.update(parse_cpp_block(block, l))

  return d

def parse_cpp_file(filename):

  def strip_nl_tab(buf):
    new_buf = ""
    for i in buf:
      if i != '\n' and i != '\t':
        new_buf += i
      else:
        new_buf += " "
    return new_buf

  fh = open(filename, "r")
  buf = fh.read()
  fh.close()

  buf = strip_nl_tab(buf)

  d = parse_cpp_block(buf, [])

  includes = p_include.split(buf)

  for i in xrange(0, len(includes)/5):
    fn = includes[(i*5)+3]
    if fn not in included_files:
      for l in include_dirs:
        try:
          d.update(parse_cpp_file(l + "/" + fn))
          included_files.append(fn)
        except:
          pass

  return d

def parse_nta_block(x):
  d = {}
  i = 0
  j = 0
  blocks = []

  while i < len(x):

    if x[i] == ',':
      blocks.append(x[j:i])
      j = i+1
    if (x[i] == '!' and x[i+1] == '<') or x[i] == '<':
      if x[i] == '<':
        tmp = i+1
      else:
        tmp = i+2

      count = 1
      while count > 0:
        if (x[tmp] == '!' and x[tmp+1] == '<') or x[tmp] == '<':
          count += 1
        if (x[tmp] == '!' and x[tmp+1] == '>') or x[tmp] == '>':
          count -= 1
        tmp = tmp+1

      if x[i] == '<':
        i = tmp-1
      else:
        i = tmp
                          
    i+=1

  blocks.append(x[j:])

  for i in blocks:
    j = i.strip()
    arr = j.split('=', 1)

    key = arr[0].strip()
    val = arr[1].strip()

    q = val.find("!<")
    if q == -1:
      d[key] = (val, {})
    else:
      w = val.rfind("!>")
      d[key] = (val[:q], parse_nta_block(val[q+2:w]))
  return d



def generate_code(a, d, dd):

  result = a + "<"

  for (k, v) in dd[a]:
    if d.has_key(k):
      x = d[k] 
      if x[1] != {}:
        y = generate_code(x[0], x[1], dd)
        result += y + ","
        gd[k] = y
      elif dd.has_key(x[0]):
        y = generate_code(x[0], {}, dd)
        result += y + ","
        gd[k] = y
      else:
        result += x[0] + ","
        gd[k] = x[0]
    elif gd.has_key(k):
      result += gd[k] + ","
    else:
      (a, b) = v
      if b != []:
        y = a + "<"
        for i in b:
          if gd.has_key(i):
            y += gd[i] + ","
        y += "> "
        
        result += y + ","
        gd[k] = y
      elif gd.has_key(a):
        result += gd[a] + ","
        gd[k] = gd[a]
      else:
        result += a + ","
        gd[k] = a

  result += "> \n"
  result = result.replace(",>", " >")

  return result

def parse_it(input_file):
  dd = parse_cpp_file(input_file)

  result_fn = "/tmp/nta-%s.c++" % os.path.basename(input_file )
  
  fhh = open(result_fn, "w")
  fh = open(input_file)
  while 1:
    buf = fh.readline()
  
    if buf == "":
      break

    c = buf.count("!<")
    cc = buf.count("!>")

    while cc != c:
      tmp_buf = fh.readline()
      if tmp_buf == "":
        print "Error!"
        sys.exit(1)

      c += tmp_buf.count("!<")
      cc += tmp_buf.count("!>")
      buf += tmp_buf

    if c != 0 and cc != 0:   
      x1 = buf.find('!<')
      x2 = buf.rfind('!>')

      while buf[x1] != ' ':
        x1 -= 1

      x1 += 1
      x2 += 2

      gd.clear()

      d = parse_nta_block("x=" + buf[x1:x2])['x']

      print d

      new_buf = buf[:x1]
      new_buf += generate_code(d[0], d[1], dd)
      new_buf += buf[x2:]
      fhh.write(new_buf)
    else:
      fhh.write(buf)

  fhh.close()

  return result_fn

start = time.time()

i = 1
while i < len(sys.argv):
  if sys.argv[i].startswith('-I'):
    if len(sys.argv[i]) == 2:
      include_dirs.append(sys.argv[i+1])
      i += 1
    else:
      include_dirs.append(sys.argv[i][2:])
  
  i+=1

result_fn = parse_it(sys.argv[-1])
cmd = "g++ " + " ".join(sys.argv[1:-1]) + " " + result_fn
""" bla """
gcc_start = time.time()
if not os.system(cmd):
  print "Compilation succeded (nta time: %fs, gcc time: %fs)" % (gcc_start - start, time.time() - gcc_start)
  os.unlink(result_fn)
else: 
  print "Compilation failed"

