/* 

   Predefined vectors for the vector component framework 

   Author: Bo Simonsen, November 2009
*/

#include "stl-vector.h++"
#include "vector-framework.h++"
#include "dynamic-array.h++" // defines cphstl::dynamic_array_kernel
#include "hashed-array-tree.h++" // defines cphstl::dynamic_array_kernel
#include "indirect-encapsulator.h++" // defines cphstl::dynamic_array_kernel
#include "doubly-indirect-encapsulator.h++" // defines cphstl::dynamic_array_kernel
#include "proxy-iterator.h++" // defines cphstl::rank_iterator
#include "rank-iterator.h++" // defines cphstl::rank_iterator
#include <memory>

namespace cphstl {

  template <typename V,
            typename A = std::allocator<V> >
  class fast_vector : public vector<V, A, 
                                    vector_framework<V, A, dynamic_array<V, A, direct_encapsulator<V, A>, true > >, 
                                    rank_iterator< vector_framework<V, A, dynamic_array<V, A, direct_encapsulator<V, A >, true > >, false>, 
                                    rank_iterator< vector_framework<V, A, dynamic_array<V, A, direct_encapsulator<V, A >, true > >, true> > { 
  private:
    typedef vector<V, A, vector_framework<V, A, dynamic_array<V, A, direct_encapsulator<V, A>, true > >, rank_iterator< vector_framework<V, A, dynamic_array<V, A, direct_encapsulator<V, A >, true > >, false>, rank_iterator< vector_framework<V, A, dynamic_array<V, A, direct_encapsulator<V, A >, true > >, true> > superclass;
  public:
    explicit fast_vector(A const& a = A()) : superclass(a) {
    }
    explicit fast_vector(typename superclass::size_type s, V const& v = V(), A const& a = A()) : superclass(s, v, a) {
    }
  
    template <typename K>
    fast_vector(K f, K l, A const& a = A()) : superclass(f, l, a) {
    }
  
    fast_vector(fast_vector const& v) : superclass(v) {
    }
    fast_vector& operator=(fast_vector const& v) {
      superclass::operator=(v);
      return (*this);
    }
  };

  template <typename V,
            typename A = std::allocator<V> >
  class safe_vector : public vector<V, A, 
                                    vector_framework<V, A, dynamic_array<V, A, doubly_indirect_encapsulator<V, A>, false > >, 
                                    proxy_iterator< vector_framework<V, A, dynamic_array<V, A, doubly_indirect_encapsulator<V, A >, false > >, false>, 
                                    proxy_iterator< vector_framework<V, A, dynamic_array<V, A, doubly_indirect_encapsulator<V, A >, false > >, true> > { 
  private:
    typedef vector<V, A, vector_framework<V, A, dynamic_array<V, A, doubly_indirect_encapsulator<V, A>, false > >, proxy_iterator< vector_framework<V, A, dynamic_array<V, A, doubly_indirect_encapsulator<V, A >, false > >, false>, proxy_iterator< vector_framework<V, A, dynamic_array<V, A, doubly_indirect_encapsulator<V, A >, false > >, true> > superclass;


  public:
    explicit safe_vector(A const& a = A()) : superclass(a) {
    }
    explicit safe_vector(typename superclass::size_type s, V const& v = V(), A const& a = A()) : superclass(s, v, a) {
    }
  
    template <typename K>
    safe_vector(K f, K l, A const& a = A()) : superclass(f, l, a) {
    }
  
    safe_vector(safe_vector const& v) : superclass(v) {
    }
    safe_vector& operator=(safe_vector const& v) {
      superclass::operator=(v);
      return (*this);
    }
  };

  template <typename V,
            typename A = std::allocator<V> >
  class compact_vector : public vector<V, A, 
                                       vector_framework<V, A, hashed_array_tree<V, A, direct_encapsulator<V, A> > >, 
                                       rank_iterator< vector_framework<V, A, hashed_array_tree<V, A, direct_encapsulator<V, A > > >, false>, 
                                       rank_iterator< vector_framework<V, A, hashed_array_tree<V, A, direct_encapsulator<V, A > > >, true> > { 
  private:
    typedef vector<V, A, vector_framework<V, A, hashed_array_tree<V, A, direct_encapsulator<V, A> > >, rank_iterator< vector_framework<V, A, hashed_array_tree<V, A, direct_encapsulator<V, A > > >, false>, rank_iterator< vector_framework<V, A, hashed_array_tree<V, A, direct_encapsulator<V, A > > >, true> > superclass;

  public:
    explicit compact_vector(A const& a = A()) : superclass(a) {
    }
    explicit compact_vector(typename superclass::size_type s, V const& v = V(), A const& a = A()) : superclass(s, v, a) {
    }
  
    template <typename K>
    compact_vector(K f, K l, A const& a = A()) : superclass(f, l, a) {
    }
  
    compact_vector(compact_vector const& v) : superclass(v) {
    }
    compact_vector& operator=(compact_vector const& v) {
      superclass::operator=(v);
      return (*this);
    }
  };
}
